### 
# SAX1 to SAX2 Filter 
# Robin Berjon <robin@knowscape.com> 
# (based mainly on code by Matt Sergeant <matt@sergeant.org>) 
# 23/08/2001 - v.0.01 
# 04/09/2001 - v.0.02 by Kip Hampton <khampton@totalcinema.com> 
# 07/10/2001 - v.0.03 by Kip Hampton <khampton@totalcinema.com> 
### 

package XML::Filter::SAX1toSAX2; 
use strict; 
use XML::Filter::Base; 
use XML::NamespaceSupport; 

use vars qw($VERSION @ISA); 
@ISA = qw(XML::Filter::Base); 
$VERSION = '0.03'; 

 
#,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,# 
#`,`, SAX1 Handler code `,`,`,`,`,`,`,`,`,`,`,`,`,`,`,`,`,`,`,`,`,`,# 
#```````````````````````````````````````````````````````````````````# 

#-------------------------------------------------------------------# 
# start_document 
#-------------------------------------------------------------------# 
sub start_document { 
    my $self    = shift; 
    my $doc     = shift; 

    $self->{NSHelper} = XML::NamespaceSupport->new({xmlns => 1}); 
    $self->SUPER::start_document($doc); 
} 
#-------------------------------------------------------------------# 

#-------------------------------------------------------------------# 
# start_element 
#-------------------------------------------------------------------# 
sub start_element { 
    my $self    = shift; 
    my $e       = shift; 

    my $el2 = $self->make_sax2($e); 
    push @{$self->{Current_Element}}, $el2; 
    $self->SUPER::start_element($el2); 
} 
#-------------------------------------------------------------------# 

#-------------------------------------------------------------------# 
# end_element 
#-------------------------------------------------------------------# 
sub end_element { 
    my $self    = shift; 

    my $e = pop @{$self->{Current_Element}}; 
    $self->SUPER::end_element($e); 
    my $prev_ns = pop @{$self->{Current_NS}}; 
    for my $ns (@$prev_ns) { 
        $self->SUPER::end_prefix_mapping({ Prefix => $ns->[0], NamespaceURI => $ns->[1] }); 
    } 
} 
#-------------------------------------------------------------------# 

#,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,# 
#`,`, Conversion Helpers ,`,`,`,`,`,`,`,`,`,`,`,`,`,`,`,`,`,`,`,`,`,# 
#```````````````````````````````````````````````````````````````````# 

#-------------------------------------------------------------------# 
# make_sax2($e) 
# convert a SAX1 Element to a SAX2 Element 
#-------------------------------------------------------------------# 
sub make_sax2 { 
    my $self    = shift; 
    my $e       = shift; 

    my $el2 = {}; 
    # add new namespaces to the stack 
    $self->{NSHelper}->push_context; 

    $self->_scan_namespaces($e->{Attributes}); 

    # take care of the attributes 
    my %attr; 
    for my $key (keys %{$e->{Attributes}}) { 
        my ($ns,$prefix,$lname) = $self->{NSHelper}->process_attribute_name($key); 
        $ns ||= ''; 
        $attr{"\{$ns\}$lname"} = { 
                        Name            => $key, 
                        Value           => $e->{Attributes}{$key}, 
                        NamespaceURI    => $ns, 
                        LocalName       => $lname, 
                        Prefix          => $prefix, 
                    }; 
    } 
    $el2->{Attributes} = \%attr; 

    # take care of the element 
    my ($e_ns,$e_prefix,$e_lname) = $self->{NSHelper}->process_element_name($e->{Name}); 

    $el2->{NamespaceURI}  = $e_ns; 
    $el2->{LocalName}     = $e_lname; 
    $el2->{Prefix}        = $e_prefix; 
    $el2->{Name}          = $e->{Name}; 

    return $el2; 
} 
#-------------------------------------------------------------------# 


#-------------------------------------------------------------------# 
# _scan_namespaces(\%attr) 
# add newly defined ns to the stack 
#-------------------------------------------------------------------# 
sub _scan_namespaces { 
    my $self    = shift; 
    my $attr    = shift; 

    my @new_ns; 
    while (my ($n,$v) = each %$attr) { 
        if ($n =~ m/^xmlns(:(.*))?$/) { 
            my $prefix = $2 || ''; 
            $self->{NSHelper}->declare_prefix($prefix, $v); 
            $self->SUPER::start_prefix_mapping({ Prefix => $prefix, NamespaceURI => $v }); 
            push @new_ns, [$prefix, $v]; 
        } 
    } 
    push @{$self->{Current_NS}}, \@new_ns; 
} 
#-------------------------------------------------------------------# 


1; 
#,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,# 
#`,`, Documentation `,`,`,`,`,`,`,`,`,`,`,`,`,`,`,`,`,`,`,`,`,`,`,`,# 
#```````````````````````````````````````````````````````````````````# 

=pod 

=head1 NAME 

XML::Filter::SAX1toSAX2 - Convert SAX1 events to SAX2 

=head1 SYNOPSIS 

  use XML::Filter::SAX1toSAX2; 
  # create a SAX2 handler 
  my $handler = XML::Handler::SAX2Foo->new(); 
  # filter from SAX1 to SAX2 
  my $filter = XML::Filter::SAX1toSAX2->new(Handler => $handler); 
  # SAX1 parser 
  my $parser = XML::Parser::PerlSAX->new(Handler => $filter); 
  # parse file 
  $parser->parse(Source => { SystemId => "file.xml" }); 

=head1 DESCRIPTION 

This module is a variation on XML::Filter::SAX1toSAX2. It changes attribute 
access to a HASH of nodes keyed by "{nsuri}localname" per the current PerlSAX2 
proposal and implements the start_prefix_mapping() and end_prefix_mapping() handlers. 

PerlSAX2 is described here: http://209.134.132.87/~ken/perl-xml/sax-2.0.html 
Proposal v.3 is at: http://aspn.activestate.com/ASPN/Mail/Message/perl-xml/761979 

=head1 AUTHOR 

Kip Hampton, khampton@totalcinema.com, nicked from 
Robin Berjon, robin@knowscape.com, ripped off from 
Matt Sergeant, matt@sergeant.org 

=head1 COPYRIGHT 

Copyright (c) 2001 Matt Sergeant, Robin Berjon, Kip Hampton. All rights reserved. This program is 
free software; you can redistribute it and/or modify it under the same 
terms as Perl itself. 

=head1 SEE ALSO 

XML::Parser::PerlSAX, XML::Filter::Base, XML::Filter::SAX2toSAX1 

=cut 
