package XML::Filter::SAX2toSAX1;

use strict;
use vars qw($VERSION %NS_ATTR);
use base qw(XML::Filter::Base);
use Data::Dumper;

$VERSION = '0.03';

sub start_element {
    my $self = shift;
    my $data = shift;

    my $local_attrs = \%NS_ATTR;
    my $event = { Name => $data->{Name}, Attributes => \%NS_ATTR };

    # this is for attr indexed as "{ns}lname"
    for my $at (values %{$data->{Attributes}}) {
        $event->{Attributes}->{$at->{Name}} = $at->{Value};
    }

    # this is for attr indexed as {"ns"}{"lname"}
    # for my $ns (values %{$data->{Attributes}}) {
    #     for my $at (values %$ns) {
    #         $event->{Attributes}->{$at->{Name}} = $at->{Value};
    #     }
    # }

    $self->SUPER::start_element($event);
    %NS_ATTR = ();
}

sub start_prefix_mapping {
    my $self = shift;
    my $data = shift;

    if ($data->{Prefix}) {
        $NS_ATTR{'xmlns:' . $data->{Prefix}} = $data->{NamespaceURI};
    }
    else {
        $NS_ATTR{'xmlns'} = $data->{NamespaceURI};
    }
}

1;


=pod

=head1 NAME

XML::Filter::SAX2toSAX1 - Convert SAX2 events to SAX1

=head1 SYNOPSIS

  use XML::Filter::SAX2toSAX1;
  # create a SAX1 handler
  my $handler = XML::Handler::YAWriter->new();
  # filter from SAX2 to SAX1
  my $filter = XML::Filter::SAX2toSAX1->new(Handler => $handler);
  # SAX2 parser
  my $parser = Orchard::SAXDriver::Expat->new(Handler => $filter);
  # parse file
  $parser->parse( "file.xml" );

=head1 DESCRIPTION

This module is a very simple module for creating SAX1 events from
SAX2 events. It is useful in the case where you have a SAX2 parser
but want to use a SAX1 handler or filter of some sort.

=head1 TODO

  - presently, only the simplest events are taken care of. For this
  to truly map from SAX2 to SAX1 a number of the more arcane events
  would have to be implemented as well. Here is the current status
  (where - in the two first columns means that there's no equivalent
  that I've looked into for now, and in the last column that it simply
  maps without modification, or doesn't need to be mapped).

  SAX2                        SAX1                    comment
  start_document              start_document          -
  end_document                end_document            -
  start_element               start_element           DONE
  end_element                 end_element             -
  characters                  characters              -
  start_prefix_mapping        -                       DONE
  end_prefix_mapping          -                       -
  ignorable_whitespace        ignorable_whitespace    -
  processing_instruction      processing_instruction  -
  set_document_locator        set_document_locator    -
  skipped_entity              ?                       not sure what to do yet
  element_decl                element_decl            -
  attribute_decl              attlist_decl            needs mapping of the values
  internal_entity_decl        entity_decl             needs mapping of the values
  external_entity_decl        entity_decl             needs mapping of the values
  notation_decl               notation_decl           needs mapping of the values
  unparsed_entity_decl        unparsed_entity_decl    needs mapping of the values
  resolve_entity              resolve_entity          needs mapping of the values
  start_dtd                   -                       not sure how to map
  end_dtd                     -                       not sure how to map
  start_entity                -                       not sure how to map
  end_entity                  -                       not sure how to map
  start_cdata                 start_cdata             maps to another handler
  end_cdata                   end_cdata               maps to another handler
  comment                     comment                 maps to another handler
  -                           entity_reference        ?
  -                           doctype_decl            ?
  -                           xml_decl                ?

  warning                     warning                 -
  error                       error                   -
  fatal_error                 fatal_error             -

=head1 AUTHOR

Matt Sergeant, matt@sergeant.org

=head1 SEE ALSO

XML::Parser::PerlSAX, XML::Filter::Base, XML::Filter::SAX1toSAX2

=cut
